#include <Array.au3>
#include "csv.au3"
#include "download.au3"
#include "extract.au3"
#include "utils.au3"
#include "ExitCodes.au3"

Dim $sCsvFile
Dim $sRepoDir
Dim $sTargetDir

If $CmdLine[0] >= 1 Then
   $sCsvFile = $CmdLine[1]
Else
   If (FileExists(@ScriptDir & "\addons.csv") And _
	  MsgBox($MB_YESNO, "Found CSV file", "Use add-on list (addons.csv) from " & @CRLF & @ScriptDir &"?") = $IDYES) _
   Then
	  $sCsvFile = @ScriptDir & "\addons.csv"
   Else
	  ConsoleWrite("Select add-on list (.csv)" & @CRLF)
	  $sCsvFile = FileOpenDialog("Select add-on list (.csv)", @ScriptDir, "CSV file (*.csv)", 1)
	  If $sCsvFile = "" then
		 ConsoleWrite("Cancelled by user" & @CRLF)
		 Exit $EXIT_CANCELLED
	  EndIf
   EndIf
EndIf

If $CmdLine[0] >= 2 Then
   $sRepoDir = $CmdLine[2]
Else
   ConsoleWrite("Select add-on repository folder" & @CRLF)
   $sRepoDir = FileSelectFolder("Select add-on repository folder", "", 1+2+4)
   If $sRepoDir = "" then
	  ConsoleWrite("Cancelled by user" & @CRLF)
	  Exit $EXIT_CANCELLED
   EndIf
Endif

If $CmdLine[0] >= 3 Then
   $sTargetDir = $CmdLine[3]
Else
   ConsoleWrite("Select installation folder" & @CRLF)
   $sTargetDir = FileSelectFolder("Select installation folder", "", 1+2+4)
   If $sRepoDir = "" then
	  ConsoleWrite("Cancelled by user" & @CRLF)
	  Exit $EXIT_CANCELLED
   EndIf
EndIf

ConsoleWrite("$sRepoDir = " & $sRepoDir & @CRLF)
ConsoleWrite("$sTargetDir = " & $sTargetDir & @CRLF)
ConsoleWrite("$sCsvFile = " & $sCsvFile & @CRLF)

DownloadMultipleAddOnsFromCsv($sRepoDir, $sCsvFile)
InstallMultipleAddOnsFromCsv($sTargetDir, $sRepoDir, $sCsvFile)

ConsoleWrite("Finished")

MsgBox($MB_SYSTEMMODAL, "CSV Install", "Installation finished!")

Exit $EXIT_FINISHED

Func InstallMultipleAddOnsFromCsv($sTargetDir, $sRepoDir, $sCsvFile)
   Local $aAddOns = _ParseCSV($sCsvFile)
   InstallMultipleAddOns($sTargetDir, $sRepoDir, $aAddOns)
EndFunc

Func InstallMultipleAddOns($sTargetDir, $sRepoDir, $aAddOns)
   DirCreate($sTargetDir) ; ignore error if exists

   For $i=0 To UBound($aAddOns)-1
	  Local $sUrl = $aAddOns[$i][0]
	  Local $sFile = $aAddOns[$i][1]
	  Local $sMethod = $aAddOns[$i][2]
	  Local $sExcludes = $aAddOns[$i][3]

	  Local $aExcludes
	  If $sExcludes <> "" Then
		 $aExcludes = StringSplit($sExcludes, ";", $STR_NOCOUNT)
	  Endif

	  InstallAddOn($sTargetDir, $sRepoDir & "\" & $sFile, $sMethod, $aExcludes)
   Next
EndFunc

Func InstallAddOn($sTargetDir, $sFile, $sMethod, $aExcludes)
   ConsoleWrite("INSTALLING: " & $sFile & @CRLF)
   If FileExists(@ScriptDir & "\" & $sFile & ".pre.bat") Then
	  ConsoleWrite("...PREINSTALL" & @CRLF)
	  RunWait( @ScriptDir & "\" & $sFile & ".pre.bat", $sTargetDir )
   Endif
   BackupExcluded($sTargetDir, $aExcludes)

   If StringLower($sMethod) = "extract" Then
	  ConsoleWrite("...EXTRACT" & @CRLF)
	  ExtractArchive($sFile, $sTargetDir)
   ElseIf StringLower($sMethod) = "run" Then
	  ConsoleWrite("...RUN" & @CRLF)
	  ExtractUsingInstaller($sFile, $sTargetDir)
   ElseIf StringLower($sMethod) = "extract+run" Then
	  ConsoleWrite("...EXTRACT+RUN" & @CRLF)
	  ExtractUsingInstallerFromArchive($sFile, $sTargetDir)
   ElseIf StringLower($sMethod) = "install" Then
	  ConsoleWrite("...INSTALL" & @CRLF)
	  ExtractUsingScriptedInstaller($sFile, $sTargetDir)
   ElseIf StringLower($sMethod) = "extract+install" Then
	  ConsoleWrite("...EXTRACT+INSTALL" & @CRLF)
	  ExtractUsingScriptedInstallerFromArchive($sFile, $sTargetDir)
   EndIf

   If FileExists(@ScriptDir & "\" & $sFile & ".post.bat") Then
	  ConsoleWrite("...POSTINSTALL" & @CRLF)
	  RunWait( @ScriptDir & "\" & $sFile & ".post.bat", $sTargetDir )
   Endif
   RestoreExcluded($sTargetDir, $aExcludes)
   ConsoleWrite("DONE: " & $sFile & @CRLF)
EndFunc

Global $sTempDir

Func BackupExcluded($sTargetDir, $aExcludes)
   if UBound($aExcludes) > 0 Then
	  Global $sTempDir = TempFolder()
	  For $j=0 To UBound($aExcludes)-1
		 ConsoleWrite("...BACKUP: " & $aExcludes[$j] & @CRLF)
		 FileCopy( $sTargetDir & "\" & $aExcludes[$j], $sTempdir & "\" & $aExcludes[$j])
		 ; ignore error if file does not exist
	  Next
   Endif
EndFunc

Func RestoreExcluded($sTargetDir, $aExcludes)
   Global $sTempDir
   if UBound($aExcludes) > 0 Then
	  For $j=0 To UBound($aExcludes)-1
		 ConsoleWrite("...RESTORE: " & $aExcludes[$j] & @CRLF)
		 FileDelete($sTargetDir & "\" & $aExcludes[$j])
		 FileCopy($sTempDir & "\" & $aExcludes[$j],  $sTargetDir & "\" & $aExcludes[$j])
		 ; ignore error if file does not exist
	  Next
	  DirRemove($sTempDir, 1)
   Endif
EndFunc
