#include-once

#include "Utils.au3"
#include "ExitCodes.au3"

Dim $exeDir

InstallZipTools()

Func InstallZipTools()
   $exeDir = _TempFile(@TempDir, "zip", "")
   ConsoleWrite("Installing archive tools to " & $exeDir & @CRLF)
   DirCreate($exeDir)
   FileInstall("7za.exe", $exeDir & "\7za.exe")
   FileInstall("unrar.exe", $exeDir & "\unrar.exe")
   FileInstall("unrar3.dll", $exeDir & "\unrar3.dll")
   FileInstall("installer.exe", $exeDir & "\installer.exe")
   FileInstall("elevate.exe", $exeDir & "\elevate.exe")
   OnAutoItExitRegister("DeleteZipTools")
EndFunc

Func DeleteZipTools()
   ConsoleWrite("Deleting archive tools" & @CRLF)
   DirRemove($exeDir, 1)
EndFunc

Func ExtractArchive($zipPath, $dir)
   ConsoleWrite("Extracting " & $zipPath & " to " & $dir & @CRLF)
   If Not FileExists($zipPath) Then
	  ConsoleWrite("Missing archive " & $zipPath & @CRLF)
	  MsgBox($MB_SYSTEMMODAL, "Missing file", "Missing archive " & $zipPath)
	  Exit $EXIT_MISSINGZIPFILE
   EndIf

   Local $cmd
   If StringLower(StringRight($zipPath, 4)) == ".rar" Then
	  $cmd = '"' & $exeDir & '\unrar.exe" x "' & $zipPath & '" -y "' & $dir & '"'
   Else
	  $cmd = '"' & $exeDir & '\7za.exe" x "' & $zipPath & '" -y -o"' & $dir & '"'
   EndIf

   ConsoleWrite($cmd & @CRLF)
   Local $ret = RunWait($cmd)
   If @error or $ret <> 0 Then
	  ConsoleWrite("Error while extracting!" & @CRLF)
	  MsgBox($MB_SYSTEMMODAL, "Extract error", "Archive file: " & $zipPath)
	  Exit $EXIT_UNZIPERROR
   Endif
EndFunc

Func ExtractUsingInstaller($setup, $dir)
   ConsoleWrite("Please set (" & $dir & ") as installation directory" & @CRLF)
   MsgBox($MB_SYSTEMMODAL, "Installing " & $setup, "Please set (" & $dir & ") as installation directory")
   Local $cmd
   If @OSVersion = "WIN_XP" or  @OSVersion = "WIN_XPe" Then
	  $cmd = '"' & $setup & '"'
   Else
	  $cmd = '"' & $exeDir & '\elevate.exe" -wait "' & $setup & '"'
   EndIf
   ConsoleWrite("Starting: " & $cmd & @CRLF)
   Local $ret = RunWait($cmd)
   If @error or $ret <> 0 Then
	  ConsoleWrite("Error while extracting!" & @CRLF)
	  MsgBox($MB_SYSTEMMODAL, "Extract error", "Setup file: " & $setup)
	  Exit $EXIT_UNZIPERROR
   Endif
EndFunc

Func ExtractUsingScriptedInstaller($setup, $dir)
   Local $batfile = $exeDir & '\installer.bat'
   Local $batcmd = '"' & $exeDir & '\installer.exe"' & ' "' & $setup & '" "' & $dir & '"'
   FileDelete($batfile)
   FileWriteLine($batfile, $batcmd)

   Local $cmd
   If @OSVersion = "WIN_XP" or  @OSVersion = "WIN_XPe" Then
	  $cmd = '"' & $batfile & '"'
   Else
	  $cmd = '"' & $exeDir & '\elevate.exe" -wait "' & $batfile & '"'
   EndIf
   ConsoleWrite("Starting: " & $cmd & @CRLF)
   Local $ret = RunWait($cmd)
   If @error or $ret <> 0 Then
	  ConsoleWrite("Error while extracting!" & @CRLF)
	  MsgBox($MB_SYSTEMMODAL, "Extract error", "Setup file: " & $setup)
	  Exit $EXIT_UNZIPERROR
   Endif
EndFunc

Func ExtractUsingInstallerFromArchive($zip, $dir)
   Local $tmpDir = TempFolder()
   ExtractArchive($zip, $tmpDir)
   Local $exes = _FileListToArray($tmpDir, "*.exe", 1)
   Local $exe = $tmpDir & "\" & $exes[1]
   ExtractUsingInstaller($exe, $dir)
   DirRemove($tmpDir, 1)
EndFunc

Func ExtractUsingScriptedInstallerFromArchive($zip, $dir)
   Local $tmpDir = TempFolder()
   ExtractArchive($zip, $tmpDir)
   Local $exes = _FileListToArray($tmpDir, "*.exe", 1)
   Local $exe = $tmpDir & "\" & $exes[1]
   ExtractUsingScriptedInstaller($exe, $dir)
   DirRemove($tmpDir, 1)
EndFunc
